@@ -7,9 +7,10 @@ Displays views from arbitrary slices of a DataArray
 import numpy as np
 import pyqtgraph as pg
 from pyqtgraph import dockarea
-from PyQt5 import QtGui
+from PyQt5 import QtGui, QtCore
 import xarray as xr
 
+from iautil import io
 from iautil.utilities.ui import DataArrayImageView, DataArrayPlot
 from iautil.plotting.image_tool.controller import DimensionController
 
@@ -17,7 +18,7 @@ from iautil.plotting.image_tool.controller import DimensionController
 
 class SlicingTab(QtGui.QWidget):
     """
-    Houses SlicingWidgets
+    Houses SlicingWidget objects.
     """
 
     def __init__(self, data_array: xr.DataArray, parent=None) -> None:
@@ -26,6 +27,7 @@ class SlicingTab(QtGui.QWidget):
         self.parent = parent
         self.data_array = data_array
 
+        # List of SlicingWidget objects housed in tab
         self.slicing_widgets = [
             SlicingWidget(i, self, data_array) for i in range(data_array.ndim, 1, -1)
         ]
@@ -33,6 +35,8 @@ class SlicingTab(QtGui.QWidget):
         self.layout = QtGui.QGridLayout()
         self.setLayout(self.layout)
 
+        # Connects each SlicingWidget to a parent and child SlicingWidget
+        # Then adds SlicingWidget to layout
         for i in range(len(self.slicing_widgets)):
             if i == 0:
                 self.slicing_widgets[i].set_parent(self.parent.data_array_image_view)
@@ -48,6 +52,7 @@ class SlicingTab(QtGui.QWidget):
 
             self.slicing_widgets[i].roi.child_imv = self.slicing_widgets[i].image_view
 
+            # Initially disables SlicingWidget
             self.slicing_widgets[i].disable()
 
             self.layout.addWidget(self.slicing_widgets[i])
@@ -56,28 +61,34 @@ class SlicingTab(QtGui.QWidget):
 # ----------------------------------------------------------------------------------
 
 class SlicingWidget(dockarea.DockArea):
+    """
+    A widget that visualizes a slice of the overall DataArray.
+    """
 
     def __init__(self, dim: int, tab, data_array) -> None:
         super(SlicingWidget, self).__init__()
         
+        # Parent tab and DataArrayController from overall ImageTool
         self.tab = tab
-
         self.main_controller = tab.parent.data_array_controller
         
+        # Set to non-None values in SlicingTab constructor
         self.parent, self.child = None, None
         self.data_array = data_array
 
-        # Slicing ROI
+        # Creates SlicingROI
         self.roi = SlicingROI()
         self.roi.slicing_widget = self
 
-        # Subwidgets
+        # ImageView and SlicingController creation based on dimension of DataArray
         if dim == 2:
             self.image_view = DataArrayPlot()
             self.controller = SlicingController(self, data_array)
+            self.controller.layout.addWidget(self.controller.export_btn, 1, 2)
         elif dim == 3:
             self.image_view = DataArrayImageView()
             self.controller = SlicingController(self, data_array)
+            self.controller.layout.addWidget(self.controller.export_btn, 1, 2)
         elif dim == 4:
             self.image_view = DataArrayImageView()
             self.controller = SlicingController(self, data_array)
@@ -113,6 +124,7 @@ class SlicingWidget(dockarea.DockArea):
 
         # Connections
         self.controller.center_btn.clicked.connect(self.roi.center)
+        self.controller.export_btn.clicked.connect(self.export)
         self.controller.enable_chkbx.stateChanged.connect(self.toggle_enabled)
         if dim == 4:
             self.main_controller.updated.connect(
@@ -125,6 +137,9 @@ class SlicingWidget(dockarea.DockArea):
     # ------------------------------------------------------------------------------
 
     def set_parent(self, parent):
+        """
+        Sets parent SlicingWidget and connects the ROI
+        """
         self.parent = parent
 
         if isinstance(parent, DataArrayImageView):
@@ -138,6 +153,9 @@ class SlicingWidget(dockarea.DockArea):
     # ------------------------------------------------------------------------------
 
     def set_child(self, child):
+        """
+        Sets child SlicingWidget and connects the ROI
+        """
         self.child = child
 
         self.roi.child_roi = child.roi
@@ -146,6 +164,9 @@ class SlicingWidget(dockarea.DockArea):
     # ------------------------------------------------------------------------------
 
     def toggle_enabled(self):
+        """
+        Calls function to enable/disable SlicingWidget
+        """
         if self.controller.enable_chkbx.isChecked():
             self.enable()
         else:
@@ -154,6 +175,9 @@ class SlicingWidget(dockarea.DockArea):
     # ------------------------------------------------------------------------------
 
     def enable(self):
+        """
+        Enable SlicingWidget and enables "Enable" checkbox for child widget
+        """
         self.enabled = True
 
         self.image_view.setEnabled(True)
@@ -168,6 +192,9 @@ class SlicingWidget(dockarea.DockArea):
     # ------------------------------------------------------------------------------
 
     def disable(self):
+        """
+        Disables SlicingWidget and child widget
+        """
         self.enabled = False
 
         self.image_view.clear()
@@ -179,9 +206,66 @@ class SlicingWidget(dockarea.DockArea):
             self.child.controller.enable_chkbx.setChecked(False)
             self.child.controller.setEnabled(False)
 
+    # ------------------------------------------------------------------------------
+
+    def export(self):
+        data = self.image_view.data_array.values
+        coords = []
+        labels = []
+
+        for dim_ctrl in self.controller.roi_controller.dim_ctrls:
+            coords.append(dim_ctrl.used_coords)
+            labels.append(dim_ctrl.dim_lbl.text())
+
+        export_dialog = SlicingExportDialog()
+        export_dialog.exec_()
+
+        if export_dialog is not None:
+            if export_dialog.format == ".csv":
+                io.create_csv(data, coords, labels, export_dialog.path)
+
+# ----------------------------------------------------------------------------------
+
+class SlicingExportDialog(QtGui.QDialog):
+
+    def __init__ (self):
+        super().__init__()
+
+        self.setWindowModality(QtCore.Qt.ApplicationModal)
+
+        self.format_lbl = QtGui.QLabel("Format:")
+        self.format_cbx = QtGui.QComboBox()
+        self.format_cbx.addItems([".csv"])
+        self.dialog_btnbox = QtGui.QDialogButtonBox()
+        self.dialog_btnbox.addButton("OK", QtGui.QDialogButtonBox.AcceptRole)
+
+        self.layout = QtGui.QGridLayout()
+        self.setLayout(self.layout)
+
+        self.layout.addWidget(self.format_lbl, 0, 0)
+        self.layout.addWidget(self.format_cbx, 0, 1)
+        self.layout.addWidget(self.dialog_btnbox, 1, 1)
+
+        self.dialog_btnbox.accepted.connect(self.accept)
+
+    # --------------------------------------------------------------------------
+
+    def accept(self):
+
+        """
+        Sets class variables to values in dialog and closes the dialog window.
+        """
+
+        self.format = self.format_cbx.currentText()
+        self.path = QtGui.QFileDialog.getSaveFileName(self, "", "", f"(*{self.format})")[0]
+        self.close()
+
 # ----------------------------------------------------------------------------------
 
 class SlicingROI(pg.LineSegmentROI):
+    """
+    Custom LineSegmentROI to be used within a SlicingWidget
+    """
     
     def __init__(self, position=(0,0), parent=None) -> None:
         super(SlicingROI, self).__init__(position)
@@ -193,10 +277,15 @@ class SlicingROI(pg.LineSegmentROI):
     # ------------------------------------------------------------------------------
 
     def slice_data_array(self):
+        """
+        Creates new DataArray from slice coordinates and displays in ImageView
+        """
         if self.slicing_widget.enabled:
+            # DataArray and slice from parent ImageView
             p_data_array = self.parent_imv.data_array
             p_data_array_slice = self.parent_imv.data_array_slice
 
+            # Slicing coords
             data, coords = self.getArrayRegion(
                 data=p_data_array_slice,
                 img=self.parent_imv.getImageItem(),
@@ -204,6 +293,7 @@ class SlicingROI(pg.LineSegmentROI):
             )
             x_coords, y_coords = coords.astype(int)
 
+            # For coordinate indicies outside those of image
             for i in range(len(x_coords)):
                 if x_coords[i] < 0:
                     x_coords[i] = 0
@@ -214,15 +304,17 @@ class SlicingROI(pg.LineSegmentROI):
                     y_coords[i] = 0
                 if y_coords[i] >= p_data_array.values.shape[1]:
                     y_coords[i] = p_data_array.values.shape[1] - 1
-
             self.coords = x_coords, y_coords
 
+            # Creates child data array from coords
             c_data_array = xr.concat(
                 [p_data_array[x, y] for x, y in zip(x_coords, y_coords)],
                 f"{p_data_array.dims[0]}, {p_data_array.dims[1]}"
             )
 
+            # Takes slice
             if c_data_array.ndim == 3:
+                # 3D slice based off slider value
                 c_data_array_slice = c_data_array[
                     :, :, self.slicing_widget.slider.value_slider.value()
                 ]
@@ -234,11 +326,15 @@ class SlicingROI(pg.LineSegmentROI):
                 c_data_array_slice
             )  
 
+            # Updates ROIController
             self.slicing_widget.controller.roi_controller.update_controller()
 
     # ------------------------------------------------------------------------------
 
     def center(self):
+        """
+        Centers ROI.
+        """
         p_data_array = self.parent_imv.data_array
         x_1 = p_data_array.coords[p_data_array.dims[0]].values[0]
         x_2 = p_data_array.coords[p_data_array.dims[0]].values[-1]
@@ -251,12 +347,18 @@ class SlicingROI(pg.LineSegmentROI):
     # ------------------------------------------------------------------------------
 
     def move(self, x_1, x_2, y_1, y_2):
+        """
+        Moves ROI to specific location.
+        """
         self.movePoint(self.getHandles()[0], (x_1, y_1))
         self.movePoint(self.getHandles()[1], (x_2, y_2))
 
 # ----------------------------------------------------------------------------------
 
 class SlicingController(QtGui.QWidget):
+    """
+    Houses an ROIController, and various options for SlicingWidget
+    """
 
     def __init__(self, slicing_widget, data_array) -> None:
         super(SlicingController, self).__init__(slicing_widget)
@@ -271,21 +373,23 @@ class SlicingController(QtGui.QWidget):
         self.layout = QtGui.QGridLayout()
         self.setLayout(self.layout)
 
+        # Add widgets to layout
         self.layout.addWidget(self.roi_controller, 0, 0, 1, 3)
         self.layout.addWidget(self.enable_chkbx, 1, 0)
         self.layout.addWidget(self.center_btn, 1, 1)
         #self.layout.addWidget(self.export_btn, 1, 2)
-
         self.layout.setColumnStretch(0, 1)
         self.layout.setColumnStretch(1, 1)
         self.layout.setColumnStretch(2, 1)
-
         self.layout.setRowStretch(0, 6)
         self.layout.setRowStretch(1, 1)
 
 # ----------------------------------------------------------------------------------
 
 class SlicingROIController(QtGui.QWidget):
+    """
+    Houses coordinate bounds for each dimension of ROI
+    """
 
     def __init__(self, data_array, parent=None) -> None:
         super(SlicingROIController, self).__init__()
@@ -313,6 +417,9 @@ class SlicingROIController(QtGui.QWidget):
     # ------------------------------------------------------------------------------
 
     def set_dimension_order(self):
+        """ 
+        Sets new dimension order for slice
+        """
         self.data_array = self.main_controller.data_array
         
         for i in range(self.data_array.ndim):
@@ -323,6 +430,9 @@ class SlicingROIController(QtGui.QWidget):
     # ------------------------------------------------------------------------------
 
     def update_controller(self):
+        """
+        Sets dimension bounds in controller
+        """
         try:
             if self.updating == "ROI":
                 return  
@@ -335,11 +445,11 @@ class SlicingROIController(QtGui.QWidget):
             if slice_degree == 0:
                 self.dim_ctrls[0].set_dimension(
                     0, 
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[1].set_dimension(
                     1, 
-                    ends=(self.roi.coords[1][0], self.roi.coords[1][-1])
+                    indices=self.roi.coords[1]
                 )
                 for i in range(2, self.data_array.ndim):
                     self.dim_ctrls[i].setEnabled(False)
@@ -352,20 +462,20 @@ class SlicingROIController(QtGui.QWidget):
                 self.dim_ctrls[0].set_dimension(
                     0, 
                     coords=[self.data_array.coords[self.data_array.dims[0]].values[i] for i in x1_indicies],
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[1].set_dimension(
                     1, 
                     coords=[self.data_array.coords[self.data_array.dims[1]].values[i] for i in x2_indicies],
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[2].set_dimension(
                     2, 
-                    ends=(self.roi.coords[1][0], self.roi.coords[1][-1])
+                    indices=self.roi.coords[1]
                 )
                 for i in range(3, self.data_array.ndim):
                     self.dim_ctrls[i].setEnabled(False)
-
+                
             elif slice_degree == 2:
                 x1_indicies = [self.roi.parent_roi.parent_roi.coords[0][i] for i in self.roi.parent_roi.coords[0]]
                 x2_indicies = [self.roi.parent_roi.parent_roi.coords[1][i] for i in self.roi.parent_roi.coords[0]]
@@ -375,21 +485,21 @@ class SlicingROIController(QtGui.QWidget):
                 self.dim_ctrls[0].set_dimension(
                     0, 
                     coords=[self.data_array.coords[self.data_array.dims[0]].values[i] for i in x1_indicies],
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[1].set_dimension(
                     1, 
                     coords=[self.data_array.coords[self.data_array.dims[1]].values[i] for i in x2_indicies],
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[2].set_dimension(
                     2, 
                     coords=[self.data_array.coords[self.data_array.dims[2]].values[i] for i in x3_indicies],
-                    ends=(self.roi.coords[0][0], self.roi.coords[0][-1])
+                    indices=self.roi.coords[0]
                 )
                 self.dim_ctrls[3].set_dimension(
                     3, 
-                    ends=(self.roi.coords[1][0], self.roi.coords[1][-1])
+                    indices=self.roi.coords[1]
                 )
 
             self.updating = None
@@ -399,8 +509,10 @@ class SlicingROIController(QtGui.QWidget):
     # ------------------------------------------------------------------------------
 
     def update_roi(self, dim: int):
+        """
+        Sets ROI location based on changes in the SlicingROIDimensionControllers
+        """
         try:
-
             if self.updating == "Controller":
                 return  
 
@@ -465,6 +577,7 @@ class SlicingROIController(QtGui.QWidget):
             self.updating = None
         except:
             ...
+
 # ----------------------------------------------------------------------------------
 
 class SlicingROIDimensionController(QtGui.QWidget):
@@ -476,6 +589,7 @@ class SlicingROIDimensionController(QtGui.QWidget):
         self.main_controller = self.parent.main_controller
 
         self.dim = 0
+        self.used_coords = []
 
         self.dim_lbl = QtGui.QLabel()
         self.endpoint_1_cbx = QtGui.QComboBox()
@@ -497,7 +611,7 @@ class SlicingROIDimensionController(QtGui.QWidget):
 
     # ------------------------------------------------------------------------------
 
-    def set_dimension(self, dim, coords=None, ends=None):
+    def set_dimension(self, dim, coords=None, indices=None):
         self.dim = dim
 
         self.data_array = self.main_controller.data_array
@@ -514,10 +628,13 @@ class SlicingROIDimensionController(QtGui.QWidget):
         self.endpoint_2_cbx.clear()
         self.endpoint_1_cbx.addItems(dim_coords)
         self.endpoint_2_cbx.addItems(dim_coords)
-        if ends is None:
+        if indices is None:
             self.endpoint_2_cbx.setCurrentIndex(len(dim_coords) - 1)
+            self.used_coords = dim_coords
         else:
-            self.endpoint_1_cbx.setCurrentIndex(ends[0])
-            self.endpoint_2_cbx.setCurrentIndex(ends[-1])
+            self.endpoint_1_cbx.setCurrentIndex(indices[0])
+            self.endpoint_2_cbx.setCurrentIndex(indices[-1])
+
+            self.used_coords = [dim_coords[i] for i in indices]
 
 # ----------------------------------------------------------------------------------
\ No newline at end of file
